/*
 * mkuart.c
 *
 *  Created on: 2010-09-04
 *       Autor: Mirosław Kardaś
 */
#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdlib.h>
#include <util/delay.h>


#include "mkuart.h"


// definiujemy w kocu nasz bufor UART_RxBuf
volatile char UART_RxBuf[UART_RX_BUF_SIZE];
// definiujemy indeksy okrelajce ilo danych w buforze
volatile uint8_t UART_RxHead; // indeks oznaczajcy gow wa
volatile uint8_t UART_RxTail; // indeks oznaczajcy ogon wa



// definiujemy w kocu nasz bufor UART_RxBuf
volatile char UART_TxBuf[UART_TX_BUF_SIZE];
// definiujemy indeksy okrelajce ilo danych w buforze
volatile uint8_t UART_TxHead; // indeks oznaczajcy gow wa
volatile uint8_t UART_TxTail; // indeks oznaczajcy ogon wa






void USART_Init( uint16_t baud ) {
	/* Ustawienie prdkoci */
	UBRRH = (uint8_t)(baud>>8);
	UBRRL = (uint8_t)baud;
	/* Zaczenie nadajnika I odbiornika */
	UCSRB = (1<<RXEN)|(1<<TXEN);
	/* Ustawienie format ramki: 8bitw danych, 1 bit stopu */
	UCSRC = (1<<URSEL)|(3<<UCSZ0);

	// jeli korzystamy z interefejsu RS485
	#ifdef UART_DE_PORT
		// inicjalizujemy lini sterujc nadajnikiem
		UART_DE_DIR |= UART_DE_BIT;
		UART_DE_ODBIERANIE;
	#endif

	// jeli korzystamy z interefejsu RS485
	#ifdef UART_DE_PORT
		// jeli korzystamy z interefejsu RS485 zaczamy dodatkowe przerwanie TXCIE
		UCSRB |= (1<<RXEN)|(1<<TXEN)|(1<<RXCIE)|(1<<TXCIE);
	#else
		// jeli nie  korzystamy z interefejsu RS485
		UCSRB |= (1<<RXEN)|(1<<TXEN)|(1<<RXCIE);
	#endif
}

// procedura obsugi przerwania Tx Complete, gdy zostanie opniony UDR
// kompilacja gdy uywamy RS485
#ifdef UART_DE_PORT
ISR( USART_TXC_vect ) {
	UART_DE_ODBIERANIE;	// zablokuj nadajnik RS485
}
#endif


// definiujemy funkcj dodajc jeden bajtdoz bufora cyklicznego
void uart_putc( char data ) {
	uint8_t tmp_head;

    tmp_head  = (UART_TxHead + 1) & UART_TX_BUF_MASK;

          // ptla oczekuje jeeli brak miejsca w buforze cyklicznym na kolejne znaki
    while ( tmp_head == UART_TxTail ){}

    UART_TxBuf[tmp_head] = data;
    UART_TxHead = tmp_head;

    // inicjalizujemy przerwanie wystpujce, gdy bufor jest pusty, dziki
    // czemu w dalszej czci wysyaniem danych zajmie si ju procedura
    // obsugi przerwania
    UCSRB |= (1<<UDRIE);
}


void uart_puts(char *s)		// wysya acuch z pamici RAM na UART
{
  register char c;
  while ((c = *s++)) uart_putc(c);			// dopki nie napotkasz 0 wysyaj znak
}

void uart_putint(int value, int radix)	// wysya na port szeregowy tekst
{
	char string[17];			// bufor na wynik funkcji itoa
	itoa(value, string, radix);		// konwersja value na ASCII
	uart_puts(string);			// wylij string na port szeregowy
}


// definiujemy procedur obsugi przerwania nadawczego, pobierajc dane z bufora cyklicznego
ISR( USART_UDRE_vect)  {
    // sprawdzamy czy indeksy s rne
    if ( UART_TxHead != UART_TxTail ) {
    	// obliczamy i zapamitujemy nowy indeks ogona wa (moe si zrwna z gow)
    	UART_TxTail = (UART_TxTail + 1) & UART_TX_BUF_MASK;

#ifdef UART_DE_PORT
    	UART_DE_NADAWANIE;	// przecz RS485 w tryb nadawania
#endif

    	// zwracamy bajt pobrany z bufora  jako rezultat funkcji
    	UDR = UART_TxBuf[UART_TxTail];
    } else {
	// zerujemy flag przerwania wystpujcego gdy bufor pusty
	UCSRB &= ~(1<<UDRIE);
    }
}


// definiujemy funkcj pobierajc jeden bajt z bufora cyklicznego
char uart_getc(void) {
    // sprawdzamy czy indeksy s rwne
    if ( UART_RxHead == UART_RxTail ) return 0;

    // obliczamy i zapamitujemy nowy indeks ogona wa (moe si zrwna z gow)
    UART_RxTail = (UART_RxTail + 1) & UART_RX_BUF_MASK;
    // zwracamy bajt pobrany z bufora  jako rezultat funkcji
    return UART_RxBuf[UART_RxTail];
}

// definiujemy procedur obsugi przerwania odbiorczego, zapisujc dane do bufora cyklicznego
ISR( USART_RXC_vect ) {

    uint8_t tmp_head;
    char data;

    data = UDR; //pobieramy natychmiast bajt danych z bufora sprztowego

    // obliczamy nowy indeks gowy wa
    tmp_head = ( UART_RxHead + 1) & UART_RX_BUF_MASK;

    // sprawdzamy, czy w nie zacznie zjada wasnego ogona
    if ( tmp_head == UART_RxTail ) {
    	// tutaj moemy w jaki wygodny dla nas sposb obsuy  bd spowodowany
    	// prb nadpisania danych w buforze, mogoby doj do sytuacji gdzie
    	// nasz w zaczby zjada wasny ogon
    } else {
	UART_RxHead = tmp_head; 		// zapamitujemy nowy indeks
	UART_RxBuf[tmp_head] = data; 	// wpisujemy odebrany bajt do bufora
    }
}

